function [res,kept]=relaxfit(XPKS,SIGMA,DELAY,fitdim,...
     onfly,pnsign,reslist,fitmode,mcmode,guess,RES0,kept0)
%---------------------------------------------------------------
%  df-dec-99 (modif. rfpltres)   df-jan-99 (corrected F-test)
%  df-apr-97		R E L A X F I T
%       PROGRAM FOR THE RELAXATION CURVE FITTING
%       makes peak-by-peak analysis of relaxation
%       data assuming --
%       fitind=2, one-exponential decay (2-parameter fit)
%       fitind=3, one-exponential decay (3-parameter fit)
%       fitind=0, -"-, alternate-sign decay (3-parameter fit),
%                 see Fushman et al., JMB,v.266,p.173 (1997)
%
%       onfly=0, fitting relax. curves in a dialog regime, peak-by-peak
%       onfly=1, turns off the dialog when fitting
%	onfly=2, in addition to onfly=1, turns off plotting of fitting curves
%
%       error estimation using MC simulation of the
%       experimental data or fitting parameters (covar.mtrx appr.)
%       F-test & R-test included
%
%       new of sept-96:
%       c-compiler oriented version, rewritten manual-excluding procedure
%
%       CALL:   [res,kept]=relaxfit(XPKS,SIGMA,DELAY,fitdim,...
%          	onfly,pnsign,reslist,fitmode,mcmode,guess,RES0,kept0)
%
%       INPUT: 
%               XPKS  -- [pks X npts+1] matrix, containing peak intensities, in the
%                       following order:  [xpk#, int(t1), int(t2), ... , int(tnpts)]
%               SIGMA -- a row-vector (size(DELAY)) of exper.errors in peak intensities
%               DELAY --  a row-vector [t1,t2,...,tnpts] of relax.delays
%               fitind = 0, 2, or 3 (see above)
%               onfly  = 0, 1, or 2 (see above)
%               pnsign -- a row-vector (size(DELAY)) of +1/-1 depending on the sign
%                         of the initial magnetization (for the 3pn-experiment)
%		reslist -- list of res./xpeaks to perform fitting
%			   input [] for all (default)
%		fitmode = 1 for full procedure (fit + error evaluation), 
%			= 2 if you want error eval. only (need RES),kept0)
%		mcmode  = 0 if exper.data simulation is user for error eval.
%			= 1 if fit.params are MC-simulated (visualization = off)
%			= 2 same as mcmode=1, with visualization turned on
%               guess -- a vector of starting parameters for the fit
%		RES0, kept0  -- the results from prev. fit, in case you want to 
%			 	estimate or reestimate the errors 
%
%       NOTES:
%               if SIGMA is unknown, input [] for SIGMA in order to
%                       either use sigma~intensity or keep sigma=const
%               if onfly is undefined, sets to default (konfly=0)
%               if pnsign is undefined, sets to default (pnsign=[])
%                       pnsign is necessary only for the alternate-sign decay
%		if reslist is undefined or [], sets to all residues
%		if fitmode is undefined, sets to 1 (fit + error estim.)
%		if mcmode  is undefined, sets to 1 (MC-par; vis=off)
%               if guess is undefined, sets to default ([100 0.01 15])
%
%       OUTPUT: res:npks x [res# A R dA dR [C dC] sum-of-sqs/(n-2) kksig]
%               kept: [nres x npts] mtrx of 1 for kept and 0 for excluded pnts
%
%	uses:  	rfchkvar.m, rfchkin.m, rfprep.m, rfpreppk.m, rfgetsel.m, 
%		rffmins.m, rfss.m, rfdisp.m, rfpltfit.m, rfpltrf.m, 
%		rfsigma.m, rfxclude.m, rf_rtest.m, rf_ftcmp.m, rf_ftest.m, 
%		rfpltres.m, rfmcdat.m, rfcovar.m, rfmcpar.m, sortmat.m
%
%			    
% revised by David Fushman, 01-12-99,04-04-97, 02-17-97, sept-96, dec-95
% Copyright (c) 1996-97 by David Fushman, The Rockefeller University
%
%-----------------------------------------------------------------------

format compact
%-------------------check if correct input & prepare--------------------
rfchkin
%-------------------------------one more check------------------------
[ar,npar,xpks,SIG,delay0,index0,res,kept,KSIG,ksig,kerr]=rfprep(XPKS,SIGMA,...
     DELAY,fitdim,fitmode,guess,reslist,pnsign,RES0,kept0);
if kerr==1, 
   error(' Check the input params and try again...');
end
%--------------------time to prepare the data--------------------------
opt=[0 1.e-4 1.e-4 0 0 0 0 0 0 0 0 0 0 6000];	%options for fmins
chi2_005rel=[3.8415,2.9957,2.6049,2.3719,2.2141,2.0986,2.0096,1.9384,1.8799,...
1.8307,1.7886,1.7522,1.7202,1.6918,1.6664,1.6435,1.6228,1.6038,1.5865,1.5705,...
1.5557,1.5420,1.5292,1.5173,1.5061,1.4956,1.4857,1.4763,1.4675,1.4591];
        %chi2_005rel=chi2_005/(degr.of.freedom), where chi2_005 is the solution
        %for 1-gammainc(chi2_005/2,degr.of.fr./2)=0.05

[npks,npts1]=size(xpks);
npts=npts1-1;
%------------------------let's start!-------------------------------
Fig1=1;Fig2=2;Fig3=3;Fig4=4;
if onfly==2,  Fig3=1;Fig4=2; end
fname=lower(strrep(date,'-','')); 		%file name to store/backup the results
fname=[fname,'_',num2str(fix(100*rand))];	%if WIN3.1or Matlab 4.0, set: fname='dia';
if fitmode==1,				%full program
  figure(Fig1); clf
  figure(Fig2); clf
 %-----------------------main loop----------------------
 for ii=1:npks,				%residue loop
   %-------------prepare data-------------
   [xobs0,sigma0,keep0,nobs0]=rfpreppk(xpks(ii,:),SIG,ksig,npts); 
if nobs0>fitdim,
   sigma_st=sigma0;			%need for storage
   keep=keep0;
   nobs=nobs0;
   resid=xpks(ii,1);
   pass=1;
   kxcl=0;
   kchsig=0;
   knext=0;
   while knext~=1,			%while loop
     %-------------prepare xobs, delay, sigma-------------
     [xobs,sigma,delay,index]=rfgetsel(xobs0,sigma0,delay0,index0,keep); 
     %----------------------fitting-----------------------
     [AR,sumsq]=rffmins(ar,opt,xobs,sigma,delay,fitdim,index); 
     nobs=length(xobs);	%added 01/12/99
     var=sumsq/(nobs-npar);
     if pass==1,
       xobs00=xobs;delay00=delay;sigma00=sigma;
       nobs00=nobs;var00=var;index00=index;
       pass=2;
     end
     if kchsig==1,
        sigma00=sigma; var00=var;
     end
     %-------------display (print) the results--------------
     kksig=rfdisp(resid,AR,var,npar,ksig,chi2_005rel(nobs-npar));
     if onfly~=2,
      %------------visualize (plot) the results--------------
      figure(Fig1);clf
      rfpltfit(resid,xobs,delay,index,xobs00,sigma00,delay00,index00,AR,...
		sumsq,fitdim,npar,nobs00);
      %-------------plot the residuals of fit----------------
      figure(Fig2);
      rfpltrf(resid,xobs,delay,index,xobs00,sigma00,delay00,index00,AR,...
		fitdim,npar,nobs00)
      %-----------------------the RUN test-------------------
      rf_rtest(AR,xobs,delay,fitdim,index);
     end
     if kxcl==1,
       %----------------------the F-test----------------------
       [P_ftest,F_ftest]=rf_ftcmp(var00,nobs00-npar,var,nobs-npar); %added 01/12/99
       %disp(['Pftest=',num2str(P_ftest),' Fftest=',num2str(F_ftest)]); %added 01/12/99
       %Ftest=rf_ftest(var00,nobs00-npar,var,nobs-npar);
       if P_ftest<=0.01,
  	  message='  REDUCTION IS SIGNIFICANT!';
       else
 	  message=[];
       end
       disp(['F-test:  F= ',num2str(F_ftest),' P= ',num2str(P_ftest),message]);
     end
     %----------------------the fork------------------------
     kxcl=0;kchsig=0;
     if onfly==0,
        kfork=input('next?[1-accept/2-change sigma/[0]-try again/9-skip residue]==> ');
	if isempty(kfork), kfork=0; end         %default
     else
        kfork=1;				%onfly~=0 --> bypass fork
     end
     if kfork==9, knext=1; 			%skip residue
     elseif kfork==1,				%accepted
       res(ii,2:2:2*npar+2)=[AR,sumsq];
       kept(ii,:)=keep;
       if kksig==6,
	if onfly==0,
          ksig_inp=input('chi2"s too high... should the sigma be scaled??? 1/[0] ==> ');
          if ksig_inp~=1, kksig=5; end;
        else					%onfly=1,2
          disp('sigma will be automatically rescaled');
        end					%if onfly==0
       end
       KSIG(ii)=kksig;
       knext=1;
     elseif kfork==2,				%change sigma
       %----------------------change sigma----------------------
         [sigma0,kksig]=rfsigma(ksig,xobs0,sigma_st);
	 kchsig=1;
	 keep=keep0;
     else					%man.pnt.xclusion	
 	figure(Fig1);
	title(['residue No ',num2str(resid),': manual point exclusion']);
	xlabel('mouse buttons = ( exclude | exit | exit )');
	excl=rfxclude(xobs0,delay0,keep0);
	kxcl=1;
	keep=~excl;				%exclude --> keep
     end					%if kfork==1
     hold off
   end; 					%while knext~=1 
   hold off
end;			%if nobs0>fitdim
end;	%ii-loop
%return		%DF-apr-01 to stop output
 res(:,2*npar+3)=KSIG;
 eval(['save ',fname,' res kept']);
 disp(['the results has been stored in ',fname,'.mat']); 
  %------------plot relax.rate vs. res/peak number-----------
 figure(Fig3);clf
 rfpltres(res,[]);
 disp(' ');
 disp('PRELIMINARY ANALYSIS OF THE RELAXATION CURVES IS FINISHED');
 disp('NOW YOU CAN ASSESS ERRORS IN THE DERIVED PARAMETERS');
 disp('WARNING: since this process might be time consuming, you have ');
 disp('a choice of EITHER to proceed right now OR to exit the process, ');
 disp('save the results (incl. res & kept),'); 
 disp('and to estimate the errors later');
 k_mc=input('TO EVALUATE THE ERRORS PRESS 1, TO EXIT PRESS 0 (1/[0]) ==> ');
 if k_mc==1; fitmode=2; end			%switch to MD
end						%if fitmode==1

if fitmode==2,
 if onfly==0,
  if isnan(mcmode), 		%isempty(mcmode), %|isnan(mcmode),
   while 1,
     mcmode=input('MC: exp.Data(0) or opt.Params([1] or 2-vis) simulation? ==>');
     if isempty(mcmode), mcmode=1; end		%default value
     if (mcmode==0)|(mcmode==1)|(mcmode==2), break,end
   end
  end
 else
   mcmode=1;			%default for onfly
 end
 if mcmode==2, figure(Fig4); clf; end
 %-------------------------------------------------------------------
 %       ERROR EVALUATION
 %-------------------------------------------------------------------
 mc_max=500;
 err=NaN*ones(npks,npar);
 for ii=1:npks,					%i-loop
  if ~isnan(res(ii,2:2:2*npar+2)),		%the residue wasn't skipped 
    AR=res(ii,2:2:2*npar);
    chi2min=res(ii,2*npar+2);
    kksig=res(ii,2*npar+3);
    [xobs0,sigma0,keep,nobs]=rfpreppk(xpks(ii,:),SIG,kksig,npts); 
    [xobs,sigma,delay,index]=rfgetsel(xobs0,sigma0,delay0,index0,keep); 
    varmin=chi2min/(nobs-npar);
    ALPHA=sqrt(varmin);				%ALPHA=sqrt(varmin)
    if mcmode==0,			%should i include KSIG=?? here???
      %---------------MC sim.of experimental data-----------------
      %	if inputSIGMA=[], assume SIGMAreal=sigma*ALPHA
      %----------------------------------------------------
      if kksig==0|kksig==1|kksig==6,
        sigma=sigma*ALPHA;         %scaling of sigma, if inputSIGMA=[]
      %    chi2_0=nobs-2;
      end
      [me_mc,sd_mc]=rfmcdat(AR,sigma,delay,mc_max,fitdim,index,opt);
      err(ii,:)=sd_mc;
    else   				% mcmode=1
      %-----------------MC sim.of fitting parameters-------------------
      invCov=rfcovar(AR,xobs,sigma,delay,fitdim,index);  	%inv.Covariance matrix         
       [me_mc,sd_mc]=rfmcpar(AR,varmin,invCov,mc_max,mcmode,res(ii,1));
      err(ii,:)=sd_mc;
    end    				%if mcmode-loop
    %-----------------------------------------------------------------
    %	display/print the results
    %-----------------------------------------------------------------
    message=['MC_dat: res# ',num2str(res(ii,1)),':  A: ',num2str(me_mc(1)),...
      '+/-',num2str(sd_mc(1)),' 1/R: ',num2str(1/(me_mc(2))),'+/-',...
       num2str(sd_mc(2)/me_mc(2)^2)];
    if fitdim~=2,
       message=[message,' C: ',num2str(me_mc(3)),'+/-',num2str(sd_mc(3))];
    end
    disp(message);
  end					%if ~isnan(res...
 end 					%i-loop

  res(:,3:2:2*npar+1)=err;
  eval(['save ',fname,' res kept']);
  %--------plot the relax.rate distribution incl. error bars---------
  figure(Fig3);
  rfpltres(res,err);
end 					%if fitmode==2

return
%=====================================================================
